//*************************************************************
//*************************************************************
//  DD_Templ2.cpp
//  Copyright (c) 199? by 
//  ALL RIGHTS RESERVED
//  Note:
//       This template creates a DirectDraw application that
//       executes in a window. It creates and intitializes a 
//       primary surface for GDI access, and sets a clipper to
//       this surface. The resulting window has close, resize,
//       and minimize buttons on the title bar and a system
//       menu. The program window is resizeable.
//       Some unused, general-purpose, bitmap functions are
//       included in the template. 
//*************************************************************
//*************************************************************
// Date:                          Coded by:
// Filename:                      Module name:
//                                Source file:
// Program description:
//
//*************************************************************
// Libraries and software support:
//
//*************************************************************
// Development environment:
//
//*************************************************************
// System requirements:
//
//*************************************************************
// Start date:
// Update history:
//            DATE            MODIFICATION
//
//*************************************************************
// Test history:
//  TEST PROTOCOL           DATE       TEST RESULTS
//
//*************************************************************
// Programmer comments:
//
//
//*************************************************************
//*************************************************************

#ifndef WIN32_LEAN_AND_MEAN
#define WIN32_LEAN_AND_MEAN  // Exclude unnecessary components
#endif

#include <windows.h>
#include <ddraw.h>
#include <stdio.h>
#include <stdarg.h>


//Global variables
LPDIRECTDRAW4			lpDD4 = NULL;        // DirectDraw object
LPDIRECTDRAWSURFACE4    lpDDSPrimary = NULL; // DirectDraw primary
LPDIRECTDRAWSURFACE4	lpDDSOffscreen = NULL; // Not used here
DDSURFACEDESC2          ddsd; 
LPDIRECTDRAWCLIPPER	    lpDDClipper;
HWND                    hWnd;
HRESULT                 hRet;
int                     DDOn = 0;


//*************************************
//            Prototypes
//*************************************
void ReleaseObjects(void);
HRESULT DDInitFailed(HWND, HRESULT, LPCTSTR);
LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
int DD4Interface();
HBITMAP DDLoadBitmap( LPDIRECTDRAWSURFACE4, LPSTR );
BOOL DDShowBitmap( LPDIRECTDRAWSURFACE4, HBITMAP,
				   int, int, int, int);
HRESULT DDBmapToSurf(LPDIRECTDRAWSURFACE4, HBITMAP);


//*****************************************************************
// WinMain implementation for exclusive mode DirectDraw application
//*****************************************************************
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                   LPSTR lpCmdLine, int nCmdShow)
{

    MSG             msg;

    static char szAppName[] = "DDAppName" ; // Application name


     // Defining a structure of type WNDCLASSEX
     WNDCLASSEX  wndclass ;
     wndclass.cbSize        = sizeof (wndclass) ;
     wndclass.style         = CS_HREDRAW | CS_VREDRAW;
     wndclass.lpfnWndProc   = WndProc ;
     wndclass.cbClsExtra    = 0 ;
     wndclass.cbWndExtra    = 0 ;
     wndclass.hInstance     = hInstance ;
     wndclass.hIcon         = LoadIcon (NULL, IDI_APPLICATION) ;
     wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW) ;
     wndclass.hbrBackground = (HBRUSH) GetStockObject
                              (WHITE_BRUSH) ;
     wndclass.lpszMenuName  = szAppName;
     wndclass.lpszClassName = szAppName;
     wndclass.hIconSm       = LoadIcon (NULL, IDI_APPLICATION) ;
	 
	 // Register the class
	 RegisterClassEx(&wndclass);


              
     hWnd = CreateWindowEx(0,        // Extended style
                szAppName,
                "DirectDraw in Window Template",
                WS_OVERLAPPEDWINDOW,
				CW_USEDEFAULT,
				CW_USEDEFAULT,
				CW_USEDEFAULT,
				CW_USEDEFAULT,
				NULL,            // Handle of parent
                NULL,            // Handle to menu
                hInstance,       // Application instance
                NULL);           // Additional data

    if (!hWnd)
        return FALSE;

    ShowWindow(hWnd, nCmdShow);


	//*************************************
	// Create DirectDraw object and 
	// create primary surface
	//*************************************
    // Fetch DirectDraw4 interface
	hRet = DD4Interface();
	if (hRet == 0)
		return DDInitFailed(hWnd, hRet,
		                    "QueryInterface() call failed");

    // Set cooperative level to exclusive and full screen
    hRet = lpDD4->SetCooperativeLevel(hWnd, DDSCL_NORMAL);
    if (hRet != DD_OK)
        return DDInitFailed(hWnd, hRet,
		                    "SetCooperativeLevel() call failed");

    //**********************************
	//    Create the primary surface
	//**********************************
	// ddsd is a structure of type DDSRUFACEDESC2
    ZeroMemory(&ddsd, sizeof(ddsd));  // Fill structure with zeros
	// Fill in other members
    ddsd.dwSize = sizeof(ddsd);
    ddsd.dwFlags = DDSD_CAPS ;
    ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE;

    hRet = lpDD4->CreateSurface(&ddsd, &lpDDSPrimary, NULL);
    if (hRet != DD_OK)
        return DDInitFailed(hWnd, hRet,
		                    "CreateSurface() call failed");

	
    //*********************************************
	//  Create a clipper for the primary surface
	//*********************************************
    hRet = lpDD4->CreateClipper(0, &lpDDClipper, NULL);
    if (hRet != DD_OK)
        return DDInitFailed(hWnd, hRet,
		                    "Create clipper failed");
	// Associate clipper with application window
	hRet = lpDDClipper->SetHWnd(0, hWnd);
    if (hRet != DD_OK)
		return DDInitFailed(hWnd, hRet,
		                    "Clipper not linked to application window");
    // Associate clipper with primary surface
	hRet = lpDDSPrimary->SetClipper(lpDDClipper);
    if (hRet != DD_OK)
		return DDInitFailed(hWnd, hRet,
		                    "Clipper not linked to primary surface");

	 DDOn = 1;          // Set refresh switch
    
	while (GetMessage(&msg, NULL, 0, 0))
    {
        TranslateMessage(&msg);
        DispatchMessage(&msg);
    }
    return msg.wParam;
}

//*****************************************************************
// Name: WndProc
// Desc: Window procedure
//*****************************************************************
LRESULT CALLBACK WndProc (HWND hwnd, UINT message, WPARAM wParam,
LPARAM lParam)
{


    switch (message)
    {


        case WM_KEYDOWN:
            // Handle keystroke to end execution
            switch (wParam)
            {
                case VK_ESCAPE:
                    PostMessage(hwnd, WM_CLOSE, 0, 0);
                    return 0L;
					break;
			}
            break;
 
        
		case WM_PAINT:
            
		    // if(DDOn)

			return 0;
         
        case WM_DESTROY:
            // Clean up and close the app
            ReleaseObjects();
            PostQuitMessage(0);
            return 0L;

	  }
    return DefWindowProc(hwnd, message, wParam, lParam);
}

//************************************
// Name: ReleaseObject
// Desc: Releases all objects
//************************************
static void ReleaseObjects(void)
{
    if (lpDD4 != NULL)
    {
        if (lpDDSPrimary != NULL)
        {
            lpDDSPrimary->Release();
            lpDDSPrimary = NULL;
			lpDDClipper->Release();
		 // lpDDSOffscreen->Release(); // Activate if used
        }
        lpDD4->Release();
        lpDD4 = NULL;
    }
}

//***************************************
// Name: DDInitFailed()
// Desc: This function is called if an
//       initialization call fails
//***************************************
HRESULT DDInitFailed(HWND hWnd, HRESULT hRet, LPCTSTR szError)
{
    char   szBuff[128];

    sprintf(szBuff, szError);
    ReleaseObjects();
    MessageBox(hWnd, szBuff, "DirectDraw InWin Demo", MB_OK);
    DestroyWindow(hWnd);
    return hRet;

}

//******************************************
// Name: DD4Interface()
// Desc: Finds DirectDraw4 object
// 
// POST CONDITIONS:
//   returns 0 if no DirectDraw4 interface
//   returns 1 if DirectDraw4 found
//******************************************
int DD4Interface()
{

	HRESULT         DDConnect;
	LPDIRECTDRAW	lpDD;    // Pointer to DirectDraw

	DDConnect = DirectDrawCreate ( NULL,
	           &lpDD,
			   NULL);
     
	if(DDConnect != DD_OK)
		return 0;
    // Atempt to locate DirectDraw4 interface
    DDConnect = lpDD->QueryInterface(
                IID_IDirectDraw4,
	            (LPVOID *) &lpDD4);
    if(DDConnect != S_OK)
		return 0;

    lpDD->Release();      // Release old pointer
        return 1;

}

//***************************************************************
// Name: DDShowBitmap
// Desc: Displays a bitmap using a DirectDraw device context
//
// PRE CONDITIONS:
// 1. Parameter 1 is pointer to a DirectDraw surface
//    Parameter 2 is handle to the bitmap
//    Parameters 3 and 4 are the display location
//    Parameters 5 and 6 are the bitmap dimensions 
//
// POST CONDITIONS:
// Returns TRUE if successful
// 
// ERROR CONDITIONS:
// All errors exit thorugh DDInitFailed() function
//***************************************************************
BOOL DDShowBitmap( LPDIRECTDRAWSURFACE4 lpDDS, HBITMAP hBitmap,
				   int xLocation, int yLocation,
				   int bWidth, int bHeight)
{

    HDC             hdcImage = NULL;
    HDC             hdcSurf  = NULL;
	HDC             thisDevice = NULL;

    // Create a DC and select the image into it.
    hdcImage = CreateCompatibleDC( NULL );
    SelectObject( hdcImage, hBitmap);
   
    // Get a DC for the surface.
	if(lpDDS->GetDC(&hdcSurf) != DD_OK)
		DDInitFailed(hWnd, hRet,
		            "GetDC() call failed in DDShowBitmap()");
  
    // BitBlt() is used to display bitmap
    if ( BitBlt( hdcSurf, xLocation, yLocation, bWidth,
		         bHeight, hdcImage, 0, 0, SRCCOPY )
				 == FALSE ) 
		DDInitFailed(hWnd, hRet,
		            "BitBlt() call failed in DDShowBitmap()");

    // Release device contexts
    if ( hdcSurf )
        lpDDS->ReleaseDC( hdcSurf );
    if ( hdcImage )
        DeleteDC( hdcImage );

	return TRUE;
}

//***************************************************************
// Name: DDBmapToSurf
// Desc: Moves a bitmap to a DirectDraw Surface
// PRE CONDITIONS:
// 1. Parameter 1 is pointer to a DirectDraw surface
//    Parameter 2 is handle to the bitmap
//
// POST CONDITIONS:
// Bitmap is move to surface
// Returns 1 if successful
// /
// ERROR CONDITIONS:
// All errors exit thorugh DDInitFailed() function
//***************************************************************
HRESULT DDBmapToSurf(LPDIRECTDRAWSURFACE4 pdds, HBITMAP hbm)
{
    HDC                     hdcImage;
    HDC                     hdc;
    DDSURFACEDESC2          ddsd;
    HRESULT                 hr = 1;
	BOOL					retValue;

    if (hbm == NULL || pdds == NULL)
 		DDInitFailed(hWnd, hRet,
		       "Invalid surface or bitmap in DDBmapToSurf");

   // Create compatible DC and select bitmap into it
	hdcImage = CreateCompatibleDC(NULL);
    if (!hdcImage)
 		DDInitFailed(hWnd, hRet,
		       "CreateCompatibleDC() failed in DDBmapToSurf");
    SelectObject(hdcImage, hbm);
 
    // Get size of surface
    ddsd.dwSize = sizeof(ddsd);
    ddsd.dwFlags = DDSD_HEIGHT | DDSD_WIDTH;
    pdds->GetSurfaceDesc(&ddsd);

    if ((hr = pdds->GetDC(&hdc)) != DD_OK)
 		DDInitFailed(hWnd, hRet,
		       "GetDC() failed in DDBmapToSurf");

	 retValue = BitBlt(hdc, 0, 0, ddsd.dwWidth, ddsd.dwHeight,
		               hdcImage, 0, 0, SRCCOPY);
	 // Release surface immediately
	 pdds->ReleaseDC(hdc);

	 if(retValue == FALSE)
   		DDInitFailed(hWnd, hRet,
		       "BitBlt() failed in DDBmapToSurf");

        DeleteDC(hdcImage);
    return hr;
}


