//*************************************************************
//*************************************************************
//  D3D_Template_1.cpp
//  Copyright (c) ???? by 
//  ALL RIGHTS RESERVED
//  Notes:
//       This template creates a Direct3D windowed application
//       for retained mode.
//*************************************************************
//*************************************************************
// Date:                          Coded by:
// Filename:                      Module name:
//                                Source file:
// Program description:
//
//*************************************************************
// Support software
// Libraries (referenced at link-time):
//          dxguid.lib
//          dddraw.lib
//          winmm.lib   
//          d3drm.lib
//*************************************************************
// Development environment:
//
//*************************************************************
// System requirements:
//
//*************************************************************
// Start date:
// Update history:
//            DATE            MODIFICATION
//
//*************************************************************
// Test history:
//  TEST PROTOCOL           DATE       TEST RESULTS
//
//*************************************************************
// Programmer comments:
//
//
//*************************************************************
//*************************************************************
#ifndef WIN32_LEAN_AND_MEAN
#define WIN32_LEAN_AND_MEAN  // Exclude unnecessary components
#endif

#define INITGUID		// Required for Direct3D
#include <windows.h>	// Windows header files
#include <ddraw.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdarg.h>
#include <d3drmwin.h>	// Direct3D retained mode definitions
#include "resource.h"   // For icons


// Global variables
LPDIRECT3DRM3 lpD3DRM = NULL;            // Direct3D RM object manager
LPDIRECTDRAWCLIPPER lpDDClipper = NULL;  // DirectDrawClipper object 
HWND			hWnd;

// Function prototypes 
BOOL InitD3D(HWND);
void D3DError(LPCTSTR);
LRESULT CALLBACK WndProc( HWND, UINT, WPARAM, LPARAM );

//*****************************************************************
// WinMain implementation for Direct3D retained mode execution
//*****************************************************************
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                   LPSTR lpCmdLine, int nCmdShow)
{

    MSG             msg;

    static char szAppName[] = "DDAppName" ; // Application name


     // Defining a structure of type WNDCLASSEX
     WNDCLASSEX  wndclass ;
     wndclass.cbSize        = sizeof (wndclass) ;
     wndclass.style         = CS_HREDRAW | CS_VREDRAW;
     wndclass.lpfnWndProc   = WndProc ;
     wndclass.cbClsExtra    = 0 ;
     wndclass.cbWndExtra    = 0 ;
     wndclass.hInstance     = hInstance ;
     wndclass.hIcon         = LoadIcon (hInstance,
		                                MAKEINTRESOURCE(IDI_ICON1));
     wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW) ;
     wndclass.hbrBackground = (HBRUSH) GetStockObject
                              (WHITE_BRUSH) ;
     wndclass.lpszMenuName  = szAppName;
     wndclass.lpszClassName = szAppName;
     wndclass.hIconSm       = LoadIcon (hInstance,
		                                MAKEINTRESOURCE(IDI_ICON1));
	 
	 // Register the class
	 RegisterClassEx(&wndclass);
             
     hWnd = CreateWindowEx(0,        // Extended style
                szAppName,
                "3DRM InWin Template",  // <== program name
                WS_OVERLAPPEDWINDOW,
				CW_USEDEFAULT,
				CW_USEDEFAULT,
				CW_USEDEFAULT,   // <== Initial screen size
				CW_USEDEFAULT,
				NULL,            // Handle of parent
                NULL,            // Handle to menu
                hInstance,       // Application instance
                NULL);           // Additional data

    if (!hWnd)
        return FALSE;

    // Display the window.
    ShowWindow(hWnd, nCmdShow);

	// Establish the message loop
	while (GetMessage(&msg, NULL, 0, 0))
    {
        TranslateMessage(&msg);
        DispatchMessage(&msg);
    }
    return msg.wParam;
}

//*****************************************************************
//                        Windows procedure
//*****************************************************************
LRESULT CALLBACK WndProc (HWND hwnd, UINT message, WPARAM wParam,
LPARAM lParam)
{
    switch (message) 
    {

	case WM_CREATE:
	    // Initialize global variables and create the Direct3D
		// retained mode objects
		if(!InitD3D(hwnd))
		{
			PostMessage(NULL, WM_DESTROY, 0, 0);
			break;
		}
		break;

		case WM_PAINT:
        // Program image update operations go here
		break;

        case WM_DESTROY:
            // Release objects and post WM_QUIT message to terminate
			// the application
            lpD3DRM->Release();
            lpDDClipper->Release();
			PostQuitMessage(0);
        break;

        default:
            return DefWindowProc(hwnd, message, wParam, lParam);
    }
    return 0L;
}


//********************************************************
// Name: InitD3D()
// Description: Initialize Direct3D interface 
//********************************************************
BOOL InitD3D(HWND hwnd)
{
    HRESULT retval; // Return value

    // Create the Direct3DRM object.
    LPDIRECT3DRM pD3DRMTemp;
    retval = Direct3DRMCreate(&pD3DRMTemp);
    if (FAILED(retval)) 
    {
        D3DError("Failed to create Direct3DRM.");
        return FALSE;
    }
    if( FAILED( pD3DRMTemp->QueryInterface(IID_IDirect3DRM3,
                            (void **)&lpD3DRM) ) )
    {
        pD3DRMTemp->Release();
        D3DError("Direct3DRM3 interface not found" );
        return FALSE;
    }
    pD3DRMTemp->Release();

    // Create DirectDrawClipper object
    retval = DirectDrawCreateClipper(0, &lpDDClipper, NULL);
    if (FAILED(retval)) 
    {
        D3DError("Failed to create DirectDrawClipper object");
        return FALSE;
    }
    // Attach the program Window as a clipper
    retval = lpDDClipper->SetHWnd(0, hwnd);
    if (FAILED(retval)) 
    {
        D3DError("Failed to set the window handle");
        return FALSE;
    }
return TRUE;
}

//*****************************************************************
// Name: D3DD3DError()
// Description: This function is called if critical operation fails
//*****************************************************************
void D3DError(LPCTSTR szError)
{
    char   szBuff[128];

    sprintf(szBuff, szError);
    MessageBox(NULL, szBuff, "Direct3D in Win Demo 1", MB_OK);
    PostMessage(NULL, WM_DESTROY, 0, 0);
    return;
}



